/*
 * Copyright 2009 Red Hat, Inc. Red Hat licenses this file to you under the
 * Apache License, version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.hornetq.jms.example;

import java.util.Hashtable;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.hornetq.jms.client.HornetQConnectionFactory;
import org.hornetq.jms.client.HornetQSession;
import org.jnp.interfaces.NamingContext;
import org.omg.CORBA.INITIALIZE;

/**
 * A simple JMS Queue example that creates a producer and consumer on a queue
 * and sends then receives a message.
 * 
 * @author <a href="ataylor@redhat.com">Andy Taylor</a>
 */
public class QueueExampleNewHornetQ
{
    Session session;
    Connection connection = null;
    InitialContext initialContext = null;
    MessageProducer producer = null;
    MessageConsumer messageConsumer=null;

    public static void main(final String[] args)
        throws Throwable
    {
        try
        {
            new QueueExampleNewHornetQ().runExample();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    public boolean runExample()
        throws Throwable
    {

        try
        {
            initialContext = getContext();

            Queue queue = (Queue)initialContext.lookup("/queue/testQueue");

            ConnectionFactory cf = (ConnectionFactory)initialContext.lookup("/ConnectionFactory");
            
            connection = cf.createConnection();

            session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);

            producer = session.createProducer(queue);
           
            //Send messages for a long time
            for (int i = 0; i < 100; i++)
            {
                try
                {
                    TextMessage message = session.createTextMessage("This is a text message " +i);

                    System.out.println("Sent message: " + message.getText());

                    producer.send(message);

                    connection.start();
                    
                    Thread.sleep(5000);

                }
                catch (JMSException P_jmsEx)
                {                    
                    P_jmsEx.printStackTrace();
                    if (P_jmsEx
                            .getMessage()
                            .contains("Connection failure detected. Unblocking a blocking call that will never get a response"))
                    {
                        System.out.println("Wait, so queue can be deployed in time on the server...");
                        Thread.sleep(10000);
                        initialize(cf, queue);
                    }
                }
            }
            return true;
        }
        finally
        {
            finalizing();
        }
    }

    private void initialize(ConnectionFactory cf, Queue queue) throws JMSException
    {
        if (session != null)
        {
            session.close();
        }

        session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);

        producer = session.createProducer(queue);
        
    }

    private void finalizing()
    {
        // Step 12. Be sure to close our JMS resources!
        try
        {           
            if (session != null)
            {
                session.close();
            }
            if (initialContext != null)
            {
                initialContext.close();
            }
            if (connection != null)
            {
                connection.close();
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    private static InitialContext getContext()
        throws NamingException
    {
        Hashtable<String, Object> F_environment = new Hashtable<String, Object>();
        String Fs_key;
        Object F_value;

        Fs_key = Context.INITIAL_CONTEXT_FACTORY;
        F_value = "org.jnp.interfaces.NamingContextFactory";
        F_environment.put(Fs_key, F_value);

        Fs_key = Context.URL_PKG_PREFIXES;
        F_value = "org.jboss.naming:org.jnp.interfaces";
        F_environment.put(Fs_key, F_value);

        Fs_key = NamingContext.JNP_LOCAL_ADDRESS;
        F_value = "127.0.0.1";
        F_environment.put(Fs_key, F_value);

        Fs_key = Context.PROVIDER_URL;
        F_value = "127.0.0.1:13499";
        F_environment.put(Fs_key, F_value);

        return new InitialContext(F_environment);
    }

}
